'#########################################################################
'#  SMPhT - Stepper Motor P(h)ase Tester                                 #
'#  Prosty program do identyfikacji wyprowadze, faz                     #
'#  silnikw krokowych                                                   #
'#  Mikrokontroler: ATtiny2313A, wewntrzny RC 8MHz, Enable DIV8=1MHz    #
'#  Wersja: 0v6                                                          #
'#  Plik rdowy: SMPhT_0v6.bas                                         #
'#  Data ostatniej modyfikacji: 13.01.2016                               #
'#  Autor: c4v2 Cyprian Kamil Kowalski                                   #
'#########################################################################
'|                            !!! UWAGA !!!                              |
'| Znaki, literay, wyprowadze silnika naley traktowa umownie.        |
'| Su one relatywnemu ustaleniu pocztkw i kocw uzwoje.           |
'| Przebiegi silnika s interpretowane wzgldem przebiegu z pierwszego   |
'| wyprowadzenia fazowego (nie wsplnego Com[A,B] silnikw unipolarnych) |
'| podczonego do wejcia o najwyszym oznaczeniu numerycznym           |
'| z podczonych wej. przebieg ten jest oznaczany A+. Strumie        |
'| wyjciowy pokazuje zatem zalenoci fazowe wzgldem relatywnego A+    |
'| z uwzgldnieniem zmian kierunku obrotw (zamiana znakw faz B)        |
'+-----------------------------------------------------------------------+
'| Zaimplementowane rodzaje silnikw krokowych:                          |
'| 1. Bipolarny dwu-uzwojeniowy o czterech wyprowadzeniach               |
'| 2. Unipolarny cztero-uzwojeniowy o piciu wyprowadzeniach             |
'| 3. Unipolarny/bipolarny cztero-uzwojeniowy o szeciu wyprowadzeniach  |
'| 4. Bipolarny/unipolarny cztero-uzwojeniowy o omiu wyprowadzeniach    |
'----------------------------------------------------------------------------------------
'Dyrektywy kompilatora
'----------------------------------------------------------------------------------------
$regfile = "ATtiny2313A.dat"            'Dyrektywa pliku z definicjami rejestrw mikroprocesora.
$crystal = 1000000                      'Czstotliwo zegara mikroprocesora.
$hwstack = 32
$swstack = 8
$framesize = 16

'----------------------------------------------------------------------------------------
'Konfiguracja portw
'----------------------------------------------------------------------------------------
Portb = &HFF                            'Wczenie "rezystorw" podcigajcych PORTB
DdrD = &HFF                             'Ustawienie wszystkich linii PORTD jako wyjcia
Portd = &HF7                            'Ustawienie stanu wysokiego na liniach PORTD prcz PORTD.3 (LED READY Wygaszona )
DDRA = &H03                             'Ustawienie wszystkich linii dostpnych PORTA jako wyjcia
Pcmsk = &HFF                            'Konfiguracja przerwania PCInt, wyzwalanie ze wszystkich wej PORTB
$baud = 9600                            'Prdko transmisji szeregowej

'----------------------------------------------------------------------------------------
'Deklaracje procedur
'----------------------------------------------------------------------------------------
On PCINT0 Sample                        'Deklaracja obsugi przerwania PCInt
Declare Sub Get_Windings_type          'Deklaracja procedury testu topologi uzwoje silnika (na przejcia)
Declare Sub Set_char                   'Deklaracja procedury wpisujcej kod znaku pod wyliczony indeks w buforze wstpnym

'----------------------------------------------------------------------------------------
'Deklaracje staychych
'----------------------------------------------------------------------------------------
Const Vprog = "{013}{010}SMPhT_0v6 Ready.."              'Wersja programu, prompt
Const Cerr = "Error!"                    'Staa tekstowa komunikatu bdu
Const Buffsize = &H18

'----------------------------------------------------------------------------------------
'Deklaracje zmiennych
'----------------------------------------------------------------------------------------
Dim Clk As Byte                         'Indeks zapenienia bufora wejciowego
Dim Tmp0 As Byte                        'Tymczasowa zmienna licznikowa
Dim Tmp1 As Byte                        'Tymczasowa zmienna licznikowa
Dim Tmp2 As Byte                        'Tymczasowa zmienna licznikowa
Dim Tmp3 As Byte                        'Tymczasowa zmienna licznikowa
Dim Tmp4 As Byte                        'Tymczasowa zmienna licznikowa
Dim N0pcs As Byte                       'Ilo zer do poprawnego odfiltrowania prbki z jednego bajtu bufora wejciowego
Dim Mtype As Byte                       'Warto Numeryczna rodzaju topologi uzwoje silnika
Dim Buf(Buffsize) As Byte                'Tablica bufora wejciowego
Dim Spe(&H08) As Byte                   'Tablica przechowujca odfiltrowane przebiegi napi indukowanych w uzwojeniach
Dim Con(&H08) As Byte                   'Tablica przechowujca topologi uzwoje
Dim Stmp As String * 8                  'Tymczasowy acuch znakowy

'----------------------------------------------------------------------------------------
'Program gwny
'----------------------------------------------------------------------------------------
Set Acsr.7                              'Wyczenie zasilania komparatora analogowego
PRR = &H0E                              'Wyczenie zasilania blokw T/C1, T/C0, USI
Print Vprog                             'Wysanie wersji oprogramowania i znaku zachty do USART
Enable Interrupts                      'Wczenie globalne systemu przerwa
Enable Pcint0                           'Wczenie obsugi przerwa PCInt
Portd = &HFF                            'Zawiecenie LED READY PORTD.3
Do                                      'Pocztek ptli gwnej
  $Asm
    SBI MCUCR, SE                         'Zezwolenie na przejcie w stan niskiego poboru energi
    SLEEP                                 'Przejcie mikrokontrolera w stan niskiego poboru energi (idle)
    CBI MCUCR, SE
  $End Asm
  If Clk >= Buffsize Then               'Gdy zapeniono bufor wstpny nastpuje
    Portd = &HF7                        'Wyaczenie LED READY (PORTD.3)
    Waitms 250                          'Opnienie dla rozadowania indukowanych napi w uzwojeniach silnika
    Call Get_Windings_type              'Test topologi uzwoje silnika (na przejcia)
    If Tmp0 <> &H00 Then                'Wykryte uzwojenia pasuj do zaimplementowanych typw topologi uzwoje
      Tmp4 = 0                           'Zerowanie zmiennej tymczasowej
      Tmp2 = 0                           'Zerowanie zmiennej tymczasowej
      $Asm
        CLR R16          '\
_03_LOOP:          ' For Tmp1 = 1 To Bufsize  'Pocztek ptli wykonywanej dla kadej komrki bufora wejciowego
        INC R16          ' |
        STS {tmp1}, R16  '/
      $End Asm
      If Tmp2 <> Buf(tmp1) And Buf(tmp1) <> &HFF Then       'Gdy bajt z bufora wstpnego jest rny i nie "pusty" od zapamitanego wczeniej
        Tmp0 = &H00                   'Zerowanie zmiennej tymczasowej przechowujcej ilo zer w bajcie
        $Asm
          CLR R17          '\
_01c_LOOP:         ' For Tmp3 = 0 To 7 'Ptla obliczajca ilo zer w aktualnej komrce bufora wejciowego
          STS {tmp3}, R17  '/
        $End Asm
        If Buf(tmp1).tmp3 = 0 Then  'Gdy badany bit jest zerem
          Incr Tmp0                 'Dodanie jednoci do iloci ju policzonych zer
        End If
        $Asm
          LDS R17, {tmp3}  '\
          INC R17          ' Next Tmp3  'Zwikszenie licznika ptli
          CPI R17, $08     ' |
          BRLO _01c_LOOP   '/
        $End Asm
        If Tmp0 = N0pcs And Tmp4 <= 3 Then       'wykonywane kolejno dla czterech prbek gdy "zawarto zer" w nich
          $Asm                                 'jest rwna wyliczonej dla danego typu silnika
            CLR R17          '\
_01a_LOOP:         ' For Tmp3 = 0 To 7 'Ptla wykonywana dla kadego bitu Tmp1
            STS {tmp3}, R17  '/
          $End Asm
          Spe(tmp3+1).tmp4 = Buf(tmp1).tmp3       'Przepisanie odfiltrowanych bajtw bit po bicie
          $Asm                                        'z bufora wstpnego z konwersj "poziomo - pionowo" to tablicy analizy
            LDS R17, {tmp3}    '\
            INC R17            ' Next Tmp3  'Zwikszenie licznika ptli
            CPI R17, &H08      ' |
            BRSH _01a_END_LOOP ' |
            RJMP _01a_LOOP     ' |
_01a_END_LOOP:       '/
          $End Asm
          Incr Tmp4                   'Zwikszenie licznika prbek
          Tmp2 = Buf(tmp1)            'Zapamitanie aktualnego bajtu z bufora wstpnego
        End If
      End If
      $Asm
        LDS R16, {Tmp1}       '\
        CPI R16, Buffsize     ' | +1
        BRSH _03_END_LOOP     ' Next Tmp1  'Zwikszenie licznika ptli (indeksu bufora wstpnego)
        RJMP _03_LOOP         ' |
_03_END_LOOP:           '/
      $End Asm
      Tmp4 = 0
      $Asm
        LDI R16, $08     '\
        LDI R17, $FF     ' |
        STS {Tmp3}, R17  ' |
_04_LOOP:          ' For Tmp0 = 8 To 1  'Pocztek ptli wykonywanej omiokrotnie
        STS {tmp0}, R16  '/
      $End Asm
      Tmp1 = Spe(Tmp0)
      Swap Spe(tmp0)
      Spe(Tmp0) = Tmp1 Or Spe(Tmp0)
      $Asm
        LDS R17, {Tmp3}
        CPI R17, $FF
        BRLO _END_CHECK_PHASE
        LDS R17, {Tmp1}
        CPI R17, $00
        BREQ _END_CHECK_PHASE
        CPI R17, $0F
        BREQ _END_CHECK_PHASE
_ANGLE_0:
        CPI R17, $0C
        BRNE _ANGLE_180
        CLR R17                'Kt przesunicia fazy 0
        RJMP _SET_ANGLE
_ANGLE_180:
        CPI R17, $03
        BRNE _ANGLE_90
        LDI R17, $02           'Kt przesunicia fazy 180
        RJMP _SET_ANGLE
_ANGLE_90:
        CPI R17, $06
        BRNE _ANGLE_270
        LDI R17, $01           'Kt przesunicia fazy 90
        RJMP _SET_ANGLE
_ANGLE_270:
        CPI R17, $09
        BRNE _END_CHECK_PHASE
        LDI R17, $03           'Kt przesunicia fazy 270
_SET_ANGLE:
        STS {Tmp3}, R17
_END_CHECK_PHASE:
      $End Asm
      Rotate Spe(tmp0),  Left , Tmp3
      Select Case Spe(Tmp0)          'W celu pniejszego dopisania do acucha znakw liter wyprowadze wsplnych [A, B]
        Case &HCC: Tmp2 = &H41       'oraz dopisania do acucha znakw cyfry (numeru) uzwojenia [1, 2], przypisanie ASCII 'A'
          Call Set_char     'Wywoanie procedury wpisujcej ASCII 'A' pod wyliczonym indeksem w buforze wstpnym
        Case &H66: Tmp2 = &H42       'przypisanie ASCII 'B'
          Call Set_char     'Wywoanie procedury wpisujcej ASCII 'B' pod wyliczonym indeksem w buforze wstpnym
        Case &H00: Incr Tmp4
      End Select
      $Asm
        LDS R17, {Tmp4}
        CPI R17, $03
        BRLO _PASS
        RJMP _ERROR
_PASS:
        LDS R16, {tmp0}    '\
        DEC R16            ' |
        CPI R16, $00       ' |
        BREQ _04_END_LOOP  ' Next Tmp0  'Zwikszenie licznika ptli
        RJMP _04_LOOP      ' |
_04_END_LOOP:      '/
      $End Asm
      If Mtype = &H80 Then              'Dla silnika cztero-uzwojeniowego o omiu wyprowadzeniach
        Tmp2 = &H34                     'przypisanie kodu ASCII znaku '4'
        Tmp3 = &H34                     'przypisanie kodu ASCII znaku '4'
        $Asm
          CLR R16          '\
_05_LOOP:          ' For Tmp0 = 1 To 8  'Pocztek ptli wykonywanej dla kadego indeksu tablicy analizowanych przebiegw
          INC R16          ' |
          STS {tmp0}, R16  '/
        $End Asm
        Tmp4 = Con(tmp0)              'Przepisanie zawartoci aktualnej komrki do zmiennej tymczasowej
        $Asm
          CLR R17          '\
_06_LOOP:          ' For Tmp1 = 1 To 8  'Pocztek ptli wykonywanej dla kadego indeksu tablicy analizowanych przebiegw
          INC R17          ' |
          STS {tmp1}, R17  '/
        $End Asm
        If Tmp4 = Con(tmp1) Then              'Gdy warto w znalezionej komrce ma taka sam warto jak zmienna Tmp4
          Select Case Buf(tmp1)               'Zalenie od przypisanych wczeniej kodw znakw ASCII [A, B]
            Case &H41: Buf(tmp0) = Tmp2       'Nadpisanie komrki zawartoci TMP2 (ASCII '0' dla pierwszego wykonania,
              Buf(tmp1) = Tmp2       'ASCII '3' dla ostatniego wykonania)
              Decr Tmp2              'Zwikszenie Tmp2 o jeden
            Case &H42: Buf(tmp0) = Tmp3       'Nadpisanie komrek zawartoci TMP3, ASCII '0' dla pierwszego wykonania,
              Buf(tmp1) = Tmp3       'ASCII '3' dla ostatniego wykonania)
              Decr Tmp3              'Zwikszenie Tmp3 o jeden
          End Select                         'Nadpisywane wartoci komrek po wykonaniu bloku ptli
        End If                                'przyjmuj wartoci ASCII '1' lub ASCII '3', ostatnia warto jest zamieniana niej na ASCI '2'
        $Asm
          LDS R17, {tmp1}    '\
          CPI R17, $08       ' |
          BRSH _06_END_LOOP  ' Next Tmp1  'Zwikszenie licznika ptli
          RJMP _06_LOOP      ' |
_06_END_LOOP:        '/
        $End Asm
        $Asm
          LDS R16, {tmp0}    '\
          CPI R16, $08       ' |
          BRSH _05_END_LOOP  ' Next Tmp0  'Zwikszenie licznika ptli
          RJMP _05_LOOP      ' |
_05_END_LOOP:        '/
        $End Asm
      End If
      $Asm
        LDI R16, $08     '\
_07_LOOP:          ' For Tmp0 = 1 To 8  'Pocztek ptli wykonywanej dla kadego wejcia
        STS {tmp0}, R16  '/
      $End Asm
      If Buf(tmp0) = &H33 Then        'Gdy wyliczony wczeniej znak ASCII wynosi '3'
        Buf(tmp0) = &H32              'zamiana znaku na '2' (interpretowane poniej tylko
      End If                          'dla silnika cztero uzwojeniowego)
      Select Case Spe(tmp0)           'Interpretacja prbek przebiegw
        Case &HFF: Stmp = ""         'Preparowanie acucha znakw, wejcie nie podczone
        Case &H00: Stmp = "Com"      'Preparowanie acucha znakw, wejcie podczone do odczepu wsplnego uzwojenia/uzwoje [----]
          If Mtype = &H6C Then       'Dla silnika z szecioma wyprowadzeniami
            Stmp = Stmp + Chr(buf(tmp0))       'dopisanie do acucha znakw litery wyprowadzenia wsplnego [A, B]
          End If
        Case &HCC: Stmp = "+A"       'Preparowanie acucha znakw dla przebiegu [++--]
        Case &H33: Stmp = "-A"       'Preparowanie acucha znakw dla przebiegu [--++]
        Case &H66: Stmp = "+B"       'Preparowanie acucha znakw dla przebiegu [-++-]
        Case &H99: Stmp = "-B"       'Preparowanie acucha znakw dla przebiegu [+--+]
        Case Else Goto _ERROR          'Skok do etykiety, wysanie komunikatu bdu do USART
      End Select
      If Mtype = &H80 Then            'Dla silnika cztero uzwojeniowego o omiu wyprowadzeniach
        Stmp = Stmp + Chr(buf(tmp0))  'dopisanie do acucha znakw cyfry (numeru) uzwojenia [1, 2]
      End If
      Print Tmp0 ; ": " ; Stmp   'Wysanie spreparowanego acucha znakw do USART (dla jednego wejcia)
      $Asm
        LDS R16, {tmp0}   '\
        DEC R16           ' |
        CPI R16, $00      ' |
        BREQ _07_END_LOOP ' Next Tmp0   'Zwikszenie zmiennej licznikowej ptli
        RJMP _07_LOOP     ' |
_07_END_LOOP:       '/
      $End Asm
    Else                                'Wykryte uzwojenia nie pasuj do zaimplementowanych typw topologi uzwoje
      _ERROR:
        Print Cerr                        'Wysanie komunikatu bdu do USART
    End If
    $Asm
      RJMP $0000
    $End Asm
  End If
Loop
End

'----------------------------------------------------------------------------------------
'Definicje procedur
'----------------------------------------------------------------------------------------
Sub Get_Windings_type                   'Procedura wykonujca test topologi uzwoje silnika
  Mtype = &H00                          'Zerowanie zmiennej typu silnika
  Tmp2 = &H00                           'Zerowanie zmiennej tymczasowej
  $Asm
    SER R17          '\
_00_LOOP:          ' For Tmp0 = 0 To 7  'Pocztek ptli wykonywanej omiokrotnie
    INC R17          ' |
    STS {tmp0}, R17  '/
  $End Asm
  Set Ddrb.tmp0                       'Ustawienie kolejnej linii PORTB jako wyjcie, reszta linii jako wejcia podcignite
  Reset Portb.tmp0                    'Ustawienie stanu niskiego na wyjciu
  Waitms 10                           'Opnienie ze wzgldu na indukcyjno uzwoje
  Tmp1 = &HFF                         'Ustawienie wszystkich bitw zmiennej tymczasowej w stan wysoki
  If Portb <> Pinb Then               'Jeeli miedzy PORTB.tmp0 a wejciami PINB wystpuje przejcie
    Tmp1 = Pinb                       'Przepisanie rejestru wejciowego PINB do zmiennej tymczasowej
    $Asm
      CLR R18          '\
_01_LOOP:          ' For Tmp3 = 0 To 7 'Ptla wykonywana dla kadego bitu Tmp1
      STS {tmp3}, R18  '/
    $End Asm
    If Tmp1.tmp3 = 0 Then           'Jeeli wyliczony bit jest rwny zero
      Incr Tmp2                     'Zwikszenie zmiennej przechowujcej tymczasowo sum wyprowadze
    End If                          'po wykonaniu ptli zmienna Tmp2 przechowuje (Sum_Zer_w_Prbekach_Fazowych)
    $Asm
      LDS R18, {tmp3}  '\
      INC R18          ' Next Tmp3  'Zwikszenie licznika ptli
      CPI R18, $08     ' |
      BRLO _01_LOOP    '/
    $End Asm
    Incr Mtype                        'Zwikszenie zmiennej przechowujcej tymczasowo Sum_Wyprowadze
  End If
  Con(tmp0 + 1) = Tmp1                'Przepisanie bajtu (&HFF gdy nie wykryto przejcia) do tablicy uzwoje
  Ddrb = &H00                         'Ustawienie wszystkich linii PORTB jako wejcia
  Portb = &HFF                        'Wczenie "rezystorw" podcigajcych dla wszystkich linii PORTB
  $Asm
    LDS R17, {tmp0}    '\
    CPI R17, &07       ' |
    BRSH _00_END_LOOP  ' Next Tmp0  'Zwikszenie licznika ptli
    RJMP _00_LOOP      ' |
_00_END_LOOP:        '/
  $End Asm                              'Wyliczenie typu silnika (Suma_Zer_w_Prbekach_Fazowych*Suma_Wyprowadze)
  Mtype = Mtype * Tmp2                   'Co jest rwnoznaczne dziaanu: Suma_Zer_w_Prbce_Fazowej*Suma_Wyprowadze^2
  $Asm
    LDS R16, {Mtype}       ; Wykryta konfiguracja uzwoje wskazuje na silnik:
    CPI R16, $20           ; Bipolarny dwu-uzwojeniowy o czterech wyprowadzeniach
    BRNE _MOTOR_5
    LDI R17, $02
    RJMP _SET_MOTOR_END
_MOTOR_5:                ; Unipolarny cztero-uzwojeniowy o piciu wyprowadzeniach
    CPI R16, $7D
    BRNE _MOTOR_6
    LDI R17, $03
    RJMP _SET_MOTOR_END
_MOTOR_6:
    CPI R16, $6C           ; Unipolarny/bipolarny cztero-uzwojeniowy o szeciu wyprowadzeniach
    BREQ _MOTOR_8
    CPI R16, $80           ; Bipolarny/unipolarny cztero-uzwojeniowy o omiu wyprowadzeniach
    BRNE _MOTOR_ERROR
_MOTOR_8:
    LDI R17, $04
    RJMP _SET_MOTOR_END
_MOTOR_ERROR:            ;Silnik niepoprawnie podczony (uszkodzony) lub nieznana topologia uzwoje
    CLR R16
    STS {Tmp0}, R16
_SET_MOTOR_END:          ; Ncps przechowuje liczb zer w bajcie potrzebn do filtracji
    STS {N0pcs}, R17       ; prbek z bufora wstpnego do analizowanej tablicy przebiegw
  $End Asm
End Sub

'----------------------------------------------------------------------------------------
Sub Set_char                            'Procedura wpisujca kod znaku pod wyliczony indeks w buforze wstpnym
  $Asm
    CLR R17          '\
_02_LOOP:          ' For Tmp1 = 0 To 7  'Ptla wykonywana dla kadego bitu komrki tablicy uzwoje
    STS {tmp1}, R17  '/
  $End Asm
  If Con(tmp0).tmp1 = 0 Then          'Jeeli wyliczony bit jest rwny zero
    Buf(tmp1 + 1) = Tmp2              'Nadpisanie komrki zawartoci Tmp2 znakiem ASCII
  End If                              '[A, B] pod wyliczonym indeksem w buforze wstpnym
  $Asm
    LDS R17, {tmp1}  '\
    INC R17          ' Next Tmp1  'Zwikszenie licznika ptli
    CPI R17, $08     ' |
    BRLO _02_LOOP    '/
  $End Asm
End Sub

'----------------------------------------------------------------------------------------
'Obsuga przerwa
'----------------------------------------------------------------------------------------
Sample:                                 'Obsuga przerwania PCInt gdy nastpi zmiana stanu na dowolnym wejciu PORTB
  Incr Clk                              'Zwikszenie indeksu zapenienia bufora wejciowego
  Buf(clk) = Pinb                       'Przepisanie stanu wej PORTB do kolejnej komrki bufora wstpnego
  If Clk = Buffsize Then                 'Gdy zapeniono bufor wstpny nastpuje
    Disable Pcint0                      'wyczenie obsugi przerwania PCInt
  End If                                'w celu dokonania interpretacji w ptli gwnej
Return                                  'Powrt z obsugi przerwania

'ENDE------------------------------------------------------------------------------------